

#ifndef _ALLSTUFF_H
#define _ALLSTUFF_H


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include <allegro.h>


#include <vector>
#include <algorithm>
using namespace std;




#define round(x)	int(floor((x)+0.5))


// objects.cpp


#define ID_EMPTY		0
#define ID_WALL			'#'
#define ID_APPLE		'*'
#define ID_TROLL		'&'
#define ID_SNAKE		'$'
#define ID_SNAKE_TAIL	0x101
#define ID_PLAYER		'@'



#define FL_JUMP_MOVE		(1<<0)
#define FL_NEED_MOVE		(1<<1)
#define FL_TROLL_MOVE_START	(1<<2)




struct tObject;

typedef void (*pThinkFun)(tObject *obj);


struct tObject {
	int		id;
	float	xp,yp,zp;
	int		move_dir;
	float	move_speed;
	float	move_dist;
	int 	flags;

	int		frame;
	int		fr_eye_l;
	int		fr_eye_r;
	int		fr_mouth;
	int 	state;
	int		distance;
	float	troll_time;
	int		troll_dist;
	int		troll_dir;
	int		troll_target_x;
	int		troll_target_y;
	float	draw_time;


	float		think_time;
	pThinkFun	think_fun;
	pThinkFun	end_move_fun;
	pThinkFun	draw_fun;

	
	tObject *next,*prev,*head;
};

const int DX[5]={ 0, 1, 0,-1, 0};
const int DY[5]={ 0, 0, 1, 0,-1};


extern int stage;
extern int score;
extern int high_score;
extern int app_eaten,app_total;
extern float stage_text;
extern int demo_mode;
extern int game_over;
extern int cheat_mode;



void start_new_game();
void start_demo_mode();

bool check_col(int x,int y,tObject **trg);
void remove_col(tObject *obj);
void set_col_xy(tObject *obj,int x,int y);
void set_col(tObject *obj);
tObject *create_object(int id,int xp,int yp);
void remove_all_objs();
void load_map(char *path);
void show_objects(BITMAP *dest);
void think_objects(float delta);

bool object_go(tObject *obj,int dir,tObject **oth=NULL);

void check_endgame();



// gfx.cpp

#define FRAME2INT(seq,fr)	(((seq)<<8)|(fr))
#define STR2INT(s)			FRAME2INT(s[0],s[1])
#define INT2SEQ(i)			(((i)>>8)&0xFF)
#define INT2FRAME(i)		((i)&0xFF)


#define FRAME_MASK			0x0000FFFF
#define FRAME_HFLIP			0x00010000


extern SAMPLE *samp[];


void load_all_frames();
void draw_frame(BITMAP *dest,int frame,int xp,int yp);
void play_samp(int id);




// think.cpp

void snake_tail_end_move(tObject *obj);
void snake_think(tObject *obj);
void troll_think(tObject *obj);
void player_troll_think(tObject *obj);
void apple_think(tObject *obj);
void player_end_move(tObject *obj);
void player_think(tObject *obj);



// pathfind.cpp

typedef int (*pPathFun)(int xp,int yp);


void path_enter(int xp,int yp,int dist,int fdir,pPathFun fun);
int path_find(int sx,int sy,pPathFun fun);



#endif

